// HUtility.h : interface of the HUtility class
// Set of utility methods
// All methods are public and static


#ifndef _HUTILITY_H
#define _HUTILITY_H

#include "HTools.h"


class HBaseView;

//! The HShell class is the data type of a HOOPS Shell.
/*! 
  HShell represents all the datastructures that are needed for a HOOPS Shell such as the points, the connectivity list and the normals.
*/

class HShell {
public:
	int	point_list_len;
	HPoint	*point_list;
	HVector	*normal_list;
	int	face_list_len;
	int	*face_list;
};


//! The HBSurfVertex class is the data type of a B-Surface vertex.
/*! 
  HBSurfVertex represents a B-Surface vertex and requires a knot value w in addition to x, y and z coordinates.
*/
class HBSurfVertex
{
public:
	double		x;	//!< The x-coordinate of a B-Surface vertex.
	double		y;	//!< The y-coordinate of a B-Surface vertex.
	double		z;	//!< The z-coordinate of a B-Surface vertex.
	double		w;	//!< The knot value of a B-Surface vertex.
	/*! 
		Set packs an HBSurfVertex object with coordinate and knot values.
	*/
	inline void Set(double X, double Y, double Z, double W) { x=X, y=Y, z=Z, w=W; }
};

//! The HPolyline class is the data type for a three-dimensional polyline.
/*! 
  The HPolyline class encapsulates the x, y, and z coordinates of a 3-dimensional polyline. 
  The 'points' member can be passed to native HOOPS routines such as HC_Insert_Polyline.
*/
class HPolyline 
{
public:
    int				m_PointCount;		//!< Number of points in the polyline.
    HPoint			*m_pHPointArray;	//!< A pointer to an array of HPoint objects.
    HPolyline		*m_pNextPolyline;	//!< A pointer to a single HPolyline object.
};

//! The HShellVertexData class encapsulates information associated with the vertices of a shell.  
/*! 
  HShellVertexData simplifies the management of shell data.  It is widely used by public and internal functions 
  during manipulation and editing of HOOPS shells.
*/
class HShellVertexData 
{
public:
    int         	m_VertexCount;		//!< Number of vertices in the shell.
    float       	*m_pFIndexArray;	//!< Pointer to an array of f-indices associated with the vertices.
    HC_KEY			m_ShellKey;			//!< Key to a given shell.
    char			m_sType[64];	
};


//! The HUtility class contains a set of utility functions and data structures used widely in MVO classes.
/*! 
  HUtility includes utility functions for double-precision vector math, creation of construction geometry, etc...
*/
class MVO_API HUtility
{
public:
	static const float PI;			//!<	default=3.1415926f

	enum{X, Y, Z, W};	//!<	Enumerated datatypes defining the different cartesian axes.
	enum{XY, XZ, YZ};	//!<	Enumerated datatypes defining the different cartesian planes.

	static const float EH_Epsilon;	//!<	default=0.01

	static void		Order(HPoint* one, HPoint* two);

	static float	GetTime();
	
	static void		ClampPointToWindow (HPoint *);
	static void		InsertBox (HPoint *, HPoint *);
	static void		GenerateCuboidPoints (HPoint *, HPoint *, HPoint *);
	static void		InsertRectangle (const char *, float x0, float y0, float x1, float y1);

	static void		InsertArrow(float stick_length,float tip_length, float stick_radius, float arrow_radius, const char *text, const char *stick_color, const char *tip_color,int num_sides = 6, float tip_falloff = 0.05);

	static void		ComputeDoublePrecisionCrossProduct(double * vector1, double * vector2, double * up);
	static double	ComputeDoublePrecisionDotProduct(double * vector1, double * vector2);
	static double	ComputeDoublePrecisionVectorLength(double * vector);
	static void		GetViewplaneVectors(HC_KEY segment_key, double * view, double * up);
	static void		NormalizeDoublePrecisionVector(double* vector);

	static bool		PointsEqual(HPoint *, HPoint *);
	static bool		CreateAxisCircle (HPoint & center,float radius,int axis);
	
	static void		RotateToVector(HPoint start, HPoint end);
	static void		SmoothTransition(HPoint &co,HPoint &to, HPoint &uo, float fieldwidtho, float fieldheighto, HPoint &cn, HPoint &tn, HPoint &un, float fieldwidthn, float fieldheightn, HBaseView *m_pHView);
	static void		ComputeVectorToVectorRotationMatrix(float *matrix,HPoint v1,HPoint v2);
	static bool		ExtractRGB(char *text,float &r, float &g, float &b);

	static void		TransitionQuaternion(float *quat_old, float *quat_new, float t, float *qt);
	static void		MatrixToQuaternion(float *matrix, float *quaternion);
	static void		QuaternionToMatrix(float * quaternion, float *matrix);
	static void		SmoothQuaternionTransition(HBaseView *m_pView, char *old_seg, char *new_seg, int num_of_transitions);
	static void		URIencode(const char * in, unsigned long n, char * out); //!<	out must be 3*n+1, out is returned null terminated, n=length(in)
	static void		URIdecode(const char * in, char * out, unsigned long * n);//!<	in must be null terminated. out must be >= lenght(in), n=length(out)
	static void		InsertHSFData(const char *decoded_hsf_data, unsigned int hsf_data_length);
	static void		GetCameraFromMatrix(float *matrix, HPoint *target, HPoint *position, HPoint *up_vector, HPoint *translation);
	static void		GetMatrixFromCamera(HPoint target, HPoint position, HPoint up_vector, HPoint translation, float *matrix);

 	static bool		PushAttribute(const char *attName);
	static void		PushAttribute(const char *attName, const char *attText);
	static bool		PopAttribute(const char *attName);
 	static bool		ModifyPushedAttribute(const char *attName, const char *newStyle);
	static bool		ShowPushedAttribute(const char *attName, char *attText);

	static HC_KEY	Recursive_Merge_Shells( HShell *total_info, HShell *max_info, bool do_recurse, bool delete_orig_shells = false, bool insert_null_lods = false );
	static HC_KEY	Merge_Shells( bool do_recurse );
	static void		Generate_LODs( bool do_recurse = true, bool delete_orig_shells = false, bool insert_null_lods = false, int n_lod_levels = 2, float reduction_ratio = 0.20 );
	static void		set_uniform_normal( HC_KEY key, const HVector *v );
	static bool		Segment_Shell_Allocate( HShell *total_info, HShell *max_info, bool do_recurse );
	static void		Shell_Search( HShell *total_info, HShell *max_info, bool do_recurse );

	static void		CountShellData(HC_KEY shell_key, int * faces, int * vertices, int * triangles, int * tristrips = 0);
 
	static void		FindFileNameExtension(const char * filename_in, char * ext_out);
	//!<filename_in must be null terminated. ext_out should be > strlen(filename_in) + 1, ext_out is lowercase only
	static void		FindFileNameAndDirectory(const char * filename_in, char * dir_out, char * filename_out);
	//!<filename_in must be null terminated. dir_out and filename_out should be > strlen(filename_in) + 1

private:

 	static bool		RestoreAttributeFromOption(const char *attName, char *attText);
	static void		ReplaceCharacter(char *text, char orig, char target);
 	static bool		SetAttribute(const char *attName, const char *attText);
	static bool		ShowAttribute(const char *attName, char *attText);

};

#endif



